<?php
/**
 * Tube_Post_Meta
 * 
 * Functions to display post meta on frontend, show hidden custom fields in admin
 * 
 * @package .TUBE
 * @subpackage Classes
 * @author  .TUBE gTLD <https://www.get.tube>
 * @copyright Copyright 2017, TUBE®
 * @link https://www.get.tube/wordpress
 * @since 1.0.0.0
 */
 
    
class Tube_Post_Meta {
  
  public static $instance;
    
  public static function init() {
    
    if ( is_null( self::$instance ) )
      self::$instance = new Tube_Theme();
    
    return self::$instance;
    
  }    
    
    
  // Constructor
  
  function __construct() {    
                
  }

    
  

  // Display the post meta data for the post

  function get_post_meta_output( $args = NULL) {
  
    $defaults = array(
      'classes' => array('sans-serif', 'no-bottom', 'text-muted'),
      'comment_count' => false,
      'link_author' => false,
    );
    
    $args = wp_parse_args( $args, $defaults );
    
    // filter the $args
    $args = apply_filters( 'tube_filter_post_meta_args', $args );    
    
    // local var for post_id
    $post_id = get_the_id();
    
    // classes for the post meta
    $classes = $args['classes'];
    
    // filter the classes
    $classes = apply_filters( 'tube_filter_post_meta_classes', $classes );
    
    // sanitize the classes
    foreach ( $classes as $index => $class ):
      
      $classes[ $index ]  = sanitize_html_class($class);
      
    endforeach;
      
    // turn classes into a string
    $classes = implode(' ' , $classes);
    
    
    
    // set up an array for the different parts of the post meta    
    $post_meta_parts = array();
    
    // Special icon for sticky posts
    // example of icon to return in the filter: <i class="fa fa-thumb-tack text-muted"></i>';      
    $sticky_icon = apply_filters( 'tube_filter_sticky_icon', '' );    
        
    // add sticky icon to post_meta_parts if needed    
    if ( is_sticky( $post_id ) && $sticky_icon ):
      $post_meta_parts['sticky-icon'] = $sticky_icon;
    endif;
        
      
    // add date to post_meta_parts if needed    
    
    $the_date_and_time = get_the_time( _x('l, F jS, Y, g:i a', 'Post meta: time title attribute (use PHP date / time formatting)', 'tube'), $post_id );
         
    $the_date_and_time = apply_filters( 'tube_filter_post_meta_date_and_time', $the_date_and_time );
        
    
    $the_date = get_the_date( get_option( 'date_format' ), $post_id );
         
    $the_date = apply_filters( 'tube_filter_post_meta_date', $the_date );
    
    $post_meta_parts['date'] = '<span title="' . esc_attr( $the_date_and_time ).'">' . $the_date  . '</span>';
    
    
        
    // get the author ID
    $author_id = get_the_author_meta( 'ID' );
    
    // get the author URL
    $author_url = get_author_posts_url( $author_id );    
    
    // filter for author URL
    // NOTE: This is used by the .TUBE Video Curator Plugin to wipe the author URL
    // TODO: Update curator plugin to filter on author_link
    $author_url = apply_filters( 'tube_filter_post_meta_author_url', $author_url );  
    
    // if no URL, then don't link the author
    if ( $author_url == '' ):
      
        $args['link_author'] = false;
      
    endif;
    
    // get the author Display Name
    $author_displayname = get_the_author();
    
    // filter for author display name
    // NOTE: This is used by the .TUBE Video Curator Plugin to show video creator name instead of author
    $author_displayname = apply_filters( 'tube_filter_post_meta_author_displayname', $author_displayname );   
    
    // add author to post_meta_parts if needed    
    if ( $author_displayname && $args['link_author'] ):
      
      $post_meta_parts['author'] = '<a href="'.esc_url($author_url).'">' . $author_displayname . '</a>';
      
    elseif ( $author_displayname ):
      
      $post_meta_parts['author'] = '<span>' . $author_displayname . '</span>';
    
    endif;
    
    
    // add comments to post_meta_parts if needed    
    $num_comments = get_comments_number();
    
    if ( $args['comment_count'] && ( have_comments( $post_id ) || comments_open( $post_id ) ) ) :  
      
      if ( $num_comments == 0 ):
         
        $comments = _x('No Comments', 'Post meta: No comments', 'tube');
       
      elseif ( $num_comments > 1 ):   
        
        $comments = _x('% Comments', 'Post meta: More than one comments', 'tube');
        
      else:
        
        $comments = _x('1 Comment', 'Post meta: One comment', 'tube');
          
      endif;
      
      $post_meta_parts['comments'] = '<a href="#comments-wrap" class="scroll-to-hash">' . $comments . '</a>';
       
    endif;
    
    // filter the post meta parts
    $post_meta_parts = apply_filters( 'tube_filter_post_meta_parts', $post_meta_parts );
    
          
    ob_start();
    
    if ( $post_meta_parts && is_array( $post_meta_parts ) && 0 != count( $post_meta_parts ) ):
      ?>
      <ul class="list-inline list-inline-delimited post-meta <?php echo esc_attr($classes); ?>">                  
        <?php foreach ( $post_meta_parts as $part_class => $part_value ): ?><li class="<?php echo esc_attr($part_class); ?>"><?php echo wp_kses_post($part_value); ?></li><?php  endforeach; ?>       
      </ul>
      <?php
    endif;
    
    $output = ob_get_clean();
    
    // clear out any whitespace
    $output = preg_replace(array('/[\t\n]/'), array(''), $output);
    
    // filter the post meta output
    $output = apply_filters( 'tube_filter_post_meta_output', $output );
  
    return $output;
  
  }
  
  
    
  
}